@@ -19,31 +19,6 @@ module Jekyll
         super(base)
       end
 
-      # Paths to ignore for the watch option
-      #
-      # options - A Hash of options passed to the command
-      #
-      # Returns a list of relative paths from source that should be ignored
-      def ignore_paths(options)
-        source      = options['source']
-        destination = options['destination']
-        config_files = Configuration[options].config_files(options)
-        paths = config_files + Array(destination)
-        ignored = []
-
-        source_abs = Pathname.new(source).expand_path
-        paths.each do |p|
-          path_abs = Pathname.new(p).expand_path
-          begin
-            rel_path = path_abs.relative_path_from(source_abs).to_s
-            ignored << Regexp.new(Regexp.escape(rel_path)) unless rel_path.start_with?('../')
-          rescue ArgumentError
-            # Could not find a relative path
-          end
-        end
-        ignored
-      end
-
       # Run Site#process and catch errors
       #
       # site - the Jekyll::Site object
@@ -74,15 +49,18 @@ module Jekyll
       # Returns nothing
       def add_build_options(c)
         c.option 'config',  '--config CONFIG_FILE[,CONFIG_FILE2,...]', Array, 'Custom configuration file'
+        c.option 'destination', '-d', '--destination DESTINATION', 'The current folder will be generated into DESTINATION'
+        c.option 'source', '-s', '--source SOURCE', 'Custom source directory'
         c.option 'future',  '--future', 'Publishes posts with a future date'
         c.option 'limit_posts', '--limit_posts MAX_POSTS', Integer, 'Limits the number of posts to parse and publish'
-        c.option 'watch',   '-w', '--watch', 'Watch for changes and rebuild'
+        c.option 'watch',   '-w', '--[no-]watch', 'Watch for changes and rebuild'
         c.option 'force_polling', '--force_polling', 'Force watch to use polling'
         c.option 'lsi',     '--lsi', 'Use LSI for improved related posts'
         c.option 'show_drafts',  '-D', '--drafts', 'Render posts in the _drafts folder'
         c.option 'unpublished', '--unpublished', 'Render posts that were marked as unpublished'
         c.option 'quiet',   '-q', '--quiet', 'Silence output.'
         c.option 'verbose', '-V', '--verbose', 'Print verbose output.'
+        c.option 'incremental', '-I', '--incremental', 'Enable incremental rebuild.'
       end
 
     end
