@@ -10,6 +10,7 @@ module Jekyll
             c.syntax 'serve [options]'
             c.description 'Serve your site locally'
             c.alias :server
+            c.alias :s
 
             add_build_options(c)
 
@@ -20,7 +21,8 @@ module Jekyll
             c.option 'skip_initial_build', '--skip-initial-build', 'Skips the initial site build which occurs before the server is started.'
 
             c.action do |args, options|
-              options["serving"] ||= true
+              options["serving"] = true
+              options["watch"] = true unless options.key?("watch")
               Jekyll::Commands::Build.process(options)
               Jekyll::Commands::Serve.process(options)
             end
@@ -38,7 +40,7 @@ module Jekyll
 
           s.mount(
             options['baseurl'],
-            WEBrick::HTTPServlet::FileHandler,
+            custom_file_handler,
             destination,
             file_handler_options
           )
@@ -48,7 +50,7 @@ module Jekyll
           if options['detach'] # detach the server
             pid = Process.fork { s.start }
             Process.detach(pid)
-            Jekyll.logger.info "Server detached with pid '#{pid}'.", "Run `kill -9 #{pid}' to stop the server."
+            Jekyll.logger.info "Server detached with pid '#{pid}'.", "Run `pkill -f jekyll' or `kill -9 #{pid}' to stop the server."
           else # create a new server thread, then join it with current terminal
             t = Thread.new { s.start }
             trap("INT") { s.shutdown }
@@ -74,16 +76,20 @@ module Jekyll
 
         def webrick_options(config)
           opts = {
-            :DocumentRoot       => config['destination'],
-            :Port               => config['port'],
             :BindAddress        => config['host'],
-            :MimeTypes          => mime_types,
+            :DirectoryIndex     => %w(index.html index.htm index.cgi index.rhtml index.xml),
+            :DocumentRoot       => config['destination'],
             :DoNotReverseLookup => true,
-            :StartCallback      => start_callback(config['detach']),
-            :DirectoryIndex     => %w(index.html index.htm index.cgi index.rhtml index.xml)
+            :MimeTypes          => mime_types,
+            :Port               => config['port'],
+            :StartCallback      => start_callback(config['detach'])
           }
 
-          if !config['verbose']
+          if config['verbose']
+            opts.merge!({
+              :Logger => WEBrick::Log.new($stdout, WEBrick::Log::DEBUG)
+            })
+          else
             opts.merge!({
               :AccessLog => [],
               :Logger => WEBrick::Log.new([], WEBrick::Log::WARN)
@@ -93,6 +99,21 @@ module Jekyll
           opts
         end
 
+        # Custom WEBrick FileHandler servlet for serving "/file.html" at "/file"
+        # when no exact match is found. This mirrors the behavior of GitHub
+        # Pages and many static web server configs.
+        def custom_file_handler
+          Class.new WEBrick::HTTPServlet::FileHandler do
+            def search_file(req, res, basename)
+              if file = super
+                file
+              else
+                super(req, res, "#{basename}.html")
+              end
+            end
+          end
+        end
+
         def start_callback(detached)
           unless detached
             Proc.new { Jekyll.logger.info "Server running...", "press ctrl-c to stop." }
@@ -117,9 +138,10 @@ module Jekyll
 
         # recreate NondisclosureName under utf-8 circumstance
         def file_handler_options
-          fh_option = WEBrick::Config::FileHandler
-          fh_option[:NondisclosureName] = ['.ht*','~*']
-          fh_option
+          WEBrick::Config::FileHandler.merge({
+            :FancyIndexing     => true,
+            :NondisclosureName => ['.ht*','~*']
+          })
         end
 
       end
