@@ -5,57 +5,55 @@ module Jekyll
 
     # Default options. Overridden by values in _config.yml.
     # Strings rather than symbols are used for compatibility with YAML.
-    DEFAULTS = {
+    DEFAULTS = Configuration[{
+      # Where things are
       'source'        => Dir.pwd,
       'destination'   => File.join(Dir.pwd, '_site'),
-      'plugins'       => '_plugins',
-      'layouts'       => '_layouts',
-      'data_source'   =>  '_data',
-      'keep_files'    => ['.git','.svn'],
-      'gems'          => [],
-      'collections'   => nil,
-
-      'timezone'      => nil,           # use the local timezone
-
-      'encoding'      => 'utf-8',       # always use utf-8 encoding. NEVER FORGET
+      'plugins_dir'   => '_plugins',
+      'layouts_dir'   => '_layouts',
+      'data_dir'      => '_data',
+      'includes_dir'  => '_includes',
+      'collections'   => {},
 
+      # Handling Reading
       'safe'          => false,
-      'detach'        => false,          # default to not detaching the server
-      'show_drafts'   => nil,
-      'limit_posts'   => 0,
-      'lsi'           => false,
-      'future'        => true,           # remove and make true just default
-      'unpublished'   => false,
-
-      'relative_permalinks' => false,
-
-      'markdown'      => 'kramdown',
-      'highlighter'   => 'pygments',
-      'permalink'     => 'date',
-      'baseurl'       => '',
       'include'       => ['.htaccess'],
       'exclude'       => [],
-      'paginate_path' => '/page:num',
-
+      'keep_files'    => ['.git','.svn'],
+      'encoding'      => 'utf-8',
       'markdown_ext'  => 'markdown,mkdown,mkdn,mkd,md',
-      'textile_ext'   => 'textile',
+
+      # Filtering Content
+      'show_drafts'   => nil,
+      'limit_posts'   => 0,
+      'future'        => false,
+      'unpublished'   => false,
+
+      # Plugins
+      'whitelist'     => [],
+      'gems'          => [],
+
+      # Conversion
+      'markdown'      => 'kramdown',
+      'highlighter'   => 'rouge',
+      'lsi'           => false,
+      'excerpt_separator' => "\n\n",
+      'incremental'   => false,
+
+      # Serving
+      'detach'        => false,          # default to not detaching the server
+      'port'          => '4000',
+      'host'          => '127.0.0.1',
+      'baseurl'       => '',
+
+      # Output Configuration
+      'permalink'     => 'date',
+      'paginate_path' => '/page:num',
+      'timezone'      => nil,           # use the local timezone
 
       'quiet'         => false,
-      'port'          => '4000',
-      'host'          => '0.0.0.0',
-
-      'excerpt_separator' => "\n\n",
-
-      'defaults'     => [],
-
-      'maruku' => {
-        'use_tex'    => false,
-        'use_divs'   => false,
-        'png_engine' => 'blahtex',
-        'png_dir'    => 'images/latex',
-        'png_url'    => '/images/latex',
-        'fenced_code_blocks' => true
-      },
+      'verbose'       => false,
+      'defaults'      => [],
 
       'rdiscount' => {
         'extensions' => []
@@ -66,12 +64,12 @@ module Jekyll
       },
 
       'kramdown' => {
-        'auto_ids'      => true,
-        'footnote_nr'   => 1,
-        'entity_output' => 'as_char',
-        'toc_levels'    => '1..6',
-        'smart_quotes'  => 'lsquo,rsquo,ldquo,rdquo',
-        'use_coderay'   => false,
+        'auto_ids'       => true,
+        'footnote_nr'    => 1,
+        'entity_output'  => 'as_char',
+        'toc_levels'     => '1..6',
+        'smart_quotes'   => 'lsquo,rsquo,ldquo,rdquo',
+        'enable_coderay' => false,
 
         'coderay' => {
           'coderay_wrap'              => 'div',
@@ -81,12 +79,8 @@ module Jekyll
           'coderay_bold_every'        => 10,
           'coderay_css'               => 'style'
         }
-      },
-
-      'redcloth' => {
-        'hard_breaks' => true
       }
-    }
+    }]
 
     # Public: Turn all keys into string
     #
@@ -95,24 +89,35 @@ module Jekyll
       reduce({}) { |hsh,(k,v)| hsh.merge(k.to_s => v) }
     end
 
+    def get_config_value_with_override(config_key, override)
+      override[config_key] || self[config_key] || DEFAULTS[config_key]
+    end
+
     # Public: Directory of the Jekyll source folder
     #
     # override - the command-line options hash
     #
     # Returns the path to the Jekyll source directory
     def source(override)
-      override['source'] || self['source'] || DEFAULTS['source']
+      get_config_value_with_override('source', override)
     end
 
-    def quiet?(override = {})
-      override['quiet'] || self['quiet'] || DEFAULTS['quiet']
+    def quiet(override = {})
+      get_config_value_with_override('quiet', override)
     end
+    alias_method :quiet?, :quiet
+
+    def verbose(override = {})
+      get_config_value_with_override('verbose', override)
+    end
+    alias_method :verbose?, :verbose
 
     def safe_load_file(filename)
       case File.extname(filename)
-      when '.toml'
+      when /\.toml/i
+        Jekyll::External.require_with_graceful_fail('toml') unless defined?(TOML)
         TOML.load_file(filename)
-      when /\.y(a)?ml/
+      when /\.ya?ml/i
         SafeYAML.load_file(filename)
       else
         raise ArgumentError, "No parser for '#{filename}' is available. Use a .toml or .y(a)ml file instead."
@@ -125,14 +130,14 @@ module Jekyll
     #
     # Returns an Array of config files
     def config_files(override)
-      # Be quiet quickly.
-      Jekyll.logger.log_level = :error if quiet?(override)
+      # Adjust verbosity quickly
+      Jekyll.logger.adjust_verbosity(:quiet => quiet?(override), :verbose => verbose?(override))
 
       # Get configuration from <source>/_config.yml or <source>/<config_file>
       config_files = override.delete('config')
       if config_files.to_s.empty?
         default = %w[yml yaml].find(Proc.new { 'yml' }) do |ext|
-          File.exists? Jekyll.sanitized_path(source(override), "_config.#{ext}")
+          File.exist?(Jekyll.sanitized_path(source(override), "_config.#{ext}"))
         end
         config_files = Jekyll.sanitized_path(source(override), "_config.#{default}")
         @default_config_file = true
@@ -181,7 +186,7 @@ module Jekyll
         $stderr.puts "#{err}"
       end
 
-      configuration.fix_common_issues.backwards_compatibilize
+      configuration.fix_common_issues.backwards_compatibilize.add_default_collections
     end
 
     # Public: Split a CSV string into an array containing its values
@@ -200,32 +205,28 @@ module Jekyll
     def backwards_compatibilize
       config = clone
       # Provide backwards-compatibility
-      if config.has_key?('auto') || config.has_key?('watch')
-        Jekyll.logger.warn "Deprecation:", "Auto-regeneration can no longer" +
+      if config.key?('auto') || config.key?('watch')
+        Jekyll::Deprecator.deprecation_message "Auto-regeneration can no longer" +
                             " be set from your configuration file(s). Use the"+
-                            " --watch/-w command-line option instead."
+                            " --[no-]watch/-w command-line option instead."
         config.delete('auto')
         config.delete('watch')
       end
 
-      if config.has_key? 'server'
-        Jekyll.logger.warn "Deprecation:", "The 'server' configuration option" +
+      if config.key? 'server'
+        Jekyll::Deprecator.deprecation_message "The 'server' configuration option" +
                             " is no longer accepted. Use the 'jekyll serve'" +
                             " subcommand to serve your site with WEBrick."
         config.delete('server')
       end
 
-      if config.has_key? 'server_port'
-        Jekyll.logger.warn "Deprecation:", "The 'server_port' configuration option" +
-                            " has been renamed to 'port'. Please update your config" +
-                            " file accordingly."
-        # copy but don't overwrite:
-        config['port'] = config['server_port'] unless config.has_key?('port')
-        config.delete('server_port')
-      end
+      renamed_key 'server_port', 'port', config
+      renamed_key 'plugins', 'plugins_dir', config
+      renamed_key 'layouts', 'layouts_dir', config
+      renamed_key 'data_source', 'data_dir', config
 
-      if config.has_key? 'pygments'
-        Jekyll.logger.warn "Deprecation:", "The 'pygments' configuration option" +
+      if config.key? 'pygments'
+        Jekyll::Deprecator.deprecation_message "The 'pygments' configuration option" +
                             " has been renamed to 'highlighter'. Please update your" +
                             " config file accordingly. The allowed values are 'rouge', " +
                             "'pygments' or null."
@@ -236,7 +237,7 @@ module Jekyll
 
       %w[include exclude].each do |option|
         if config.fetch(option, []).is_a?(String)
-          Jekyll.logger.warn "Deprecation:", "The '#{option}' configuration option" +
+          Jekyll::Deprecator.deprecation_message "The '#{option}' configuration option" +
             " must now be specified as an array, but you specified" +
             " a string. For now, we've treated the string you provided" +
             " as a list of comma-separated values."
@@ -245,18 +246,27 @@ module Jekyll
         config[option].map!(&:to_s)
       end
 
-      if config.fetch('markdown', 'kramdown').to_s.downcase.eql?("maruku")
-        Jekyll::Deprecator.deprecation_message "You're using the 'maruku' " +
-          "Markdown processor. Maruku support has been deprecated and will " +
-          "be removed in 3.0.0. We recommend you switch to Kramdown."
+      if (config['kramdown'] || {}).key?('use_coderay')
+        Jekyll::Deprecator.deprecation_message "Please change 'use_coderay'" +
+          " to 'enable_coderay' in your configuration file."
+        config['kramdown']['use_coderay'] = config['kramdown'].delete('enable_coderay')
       end
+
+      if config.fetch('markdown', 'kramdown').to_s.downcase.eql?("maruku")
+        Jekyll.logger.abort_with "Error:", "You're using the 'maruku' " +
+          "Markdown processor, which has been removed as of 3.0.0. " +
+          "We recommend you switch to Kramdown. To do this, replace " +
+          "`markdown: maruku` with `markdown: kramdown` in your " +
+          "`_config.yml` file."
+      end
+
       config
     end
 
     def fix_common_issues
       config = clone
 
-      if config.has_key?('paginate') && (!config['paginate'].is_a?(Integer) || config['paginate'] < 1)
+      if config.key?('paginate') && (!config['paginate'].is_a?(Integer) || config['paginate'] < 1)
         Jekyll.logger.warn "Config Warning:", "The `paginate` key must be a" +
           " positive integer or nil. It's currently set to '#{config['paginate'].inspect}'."
         config['paginate'] = nil
@@ -264,5 +274,45 @@ module Jekyll
 
       config
     end
+
+    def add_default_collections
+      config = clone
+
+      return config if config['collections'].nil?
+
+      if config['collections'].is_a?(Array)
+        config['collections'] = Hash[config['collections'].map{|c| [c, {}]}]
+      end
+      config['collections']['posts'] ||= {}
+      config['collections']['posts']['output'] = true
+      config['collections']['posts']['permalink'] = style_to_permalink(config['permalink'])
+
+      config
+    end
+
+    def renamed_key(old, new, config, allowed_values = nil)
+      if config.key?(old)
+        Jekyll::Deprecator.deprecation_message "The '#{old}' configuration" +
+          "option has been renamed to '#{new}'. Please update your config " +
+          "file accordingly."
+        config[new] = config.delete(old)
+      end
+    end
+
+    private
+    def style_to_permalink(permalink_style)
+      case permalink_style.to_sym
+      when :pretty
+        "/:categories/:year/:month/:day/:title/"
+      when :none
+        "/:categories/:title.html"
+      when :date
+        "/:categories/:year/:month/:day/:title.html"
+      when :ordinal
+        "/:categories/:year/:y_day/:title.html"
+      else
+        permalink_style.to_s
+      end
+    end
   end
 end
