@@ -5,6 +5,11 @@ module Jekyll
         def initialize(config)
           require 'kramdown'
           @config = config
+          # If kramdown supported highlighter enabled, use that
+          highlighter = @config['highlighter']
+          if highlighter == 'rouge' || highlighter == 'coderay'
+            @config['kramdown']['syntax_highlighter'] ||= highlighter
+          end
         rescue LoadError
           STDERR.puts 'You are missing a library required for Markdown. Please run:'
           STDERR.puts '  $ [sudo] gem install kramdown'
@@ -13,14 +18,14 @@ module Jekyll
 
         def convert(content)
           # Check for use of coderay
-          if @config['kramdown']['use_coderay']
+          if @config['kramdown']['enable_coderay']
             %w[wrap line_numbers line_numbers_start tab_width bold_every css default_lang].each do |opt|
               key = "coderay_#{opt}"
-              @config['kramdown'][key] = @config['kramdown']['coderay'][key] unless @config['kramdown'].has_key?(key)
+              @config['kramdown'][key] = @config['kramdown']['coderay'][key] unless @config['kramdown'].key?(key)
             end
           end
 
-          Kramdown::Document.new(content, Utils.symbolize_hash_keys(@config["kramdown"])).to_html
+          Kramdown::Document.new(content, Utils.symbolize_hash_keys(@config['kramdown'])).to_html
         end
 
       end
