@@ -10,6 +10,26 @@ module Jekyll
       @site = site
     end
 
+    def update_deprecated_types(set)
+      return set unless set.key?('scope') && set['scope'].key?('type')
+
+      set['scope']['type'] = case set['scope']['type']
+      when 'page'
+        Deprecator.defaults_deprecate_type('page', 'pages')
+        'pages'
+      when 'post'
+        Deprecator.defaults_deprecate_type('post', 'posts')
+        'posts'
+      when 'draft'
+        Deprecator.defaults_deprecate_type('draft', 'drafts')
+        'drafts'
+      else
+        set['scope']['type']
+      end
+
+      set
+    end
+
     # Finds a default value for a given setting, filtered by path and type
     #
     # path - the path (relative to the source) of the page, post or :draft the default is used in
@@ -21,7 +41,7 @@ module Jekyll
       old_scope = nil
 
       matching_sets(path, type).each do |set|
-        if set['values'].has_key?(setting) && has_precedence?(old_scope, set['scope'])
+        if set['values'].key?(setting) && has_precedence?(old_scope, set['scope'])
           value = set['values'][setting]
           old_scope = set['scope']
         end
@@ -63,18 +83,29 @@ module Jekyll
     end
 
     def applies_path?(scope, path)
-      return true if scope['path'].empty?
+      return true if !scope.has_key?('path') || scope['path'].empty?
 
       scope_path = Pathname.new(scope['path'])
       Pathname.new(sanitize_path(path)).ascend do |path|
-        if path == scope_path
+        if path.to_s == scope_path.to_s
           return true
         end
       end
     end
 
+    # Determines whether the scope applies to type.
+    # The scope applies to the type if:
+    #   1. no 'type' is specified
+    #   2. the 'type' in the scope is the same as the type asked about
+    #
+    # scope - the Hash defaults set being asked about application
+    # type  - the type of the document being processed / asked about
+    #         its defaults.
+    #
+    # Returns true if either of the above conditions are satisfied,
+    #   otherwise returns false
     def applies_type?(scope, type)
-      !scope.has_key?('type') || scope['type'] == type.to_s
+      !scope.key?('type') || scope['type'].eql?(type.to_s)
     end
 
     # Checks if a given set of default values is valid
@@ -83,7 +114,7 @@ module Jekyll
     #
     # Returns true if the set is valid and can be used in this class
     def valid?(set)
-      set.is_a?(Hash) && set['scope'].is_a?(Hash) && set['scope']['path'].is_a?(String) && set['values'].is_a?(Hash)
+      set.is_a?(Hash) && set['values'].is_a?(Hash)
     end
 
     # Determines if a new scope has precedence over an old one
@@ -100,10 +131,10 @@ module Jekyll
 
       if new_path.length != old_path.length
         new_path.length >= old_path.length
-      elsif new_scope.has_key? 'type'
+      elsif new_scope.key? 'type'
         true
       else
-        !old_scope.has_key? 'type'
+        !old_scope.key? 'type'
       end
     end
 
@@ -112,7 +143,7 @@ module Jekyll
     # Returns an array of hashes
     def matching_sets(path, type)
       valid_sets.select do |set|
-        applies?(set['scope'], path, type)
+        !set.has_key?('scope') || applies?(set['scope'], path, type)
       end
     end
 
@@ -126,20 +157,23 @@ module Jekyll
       sets = @site.config['defaults']
       return [] unless sets.is_a?(Array)
 
-      sets.select do |set|
-        unless valid?(set)
-          Jekyll.logger.warn "Default:", "An invalid default set was found"
+      sets.map do |set|
+        if valid?(set)
+          update_deprecated_types(set)
+        else
+          Jekyll.logger.warn "Defaults:", "An invalid front-matter default set was found:"
+          Jekyll.logger.warn "#{set}"
+          nil
         end
-        valid?(set)
-      end
+      end.compact
     end
 
-    # Sanitizes the given path by removing a leading and addding a trailing slash
+    # Sanitizes the given path by removing a leading and adding a trailing slash
     def sanitize_path(path)
       if path.nil? || path.empty?
         ""
       else
-        path.gsub(/\A\//, '').gsub(/([^\/])\z/, '\1/')
+        path.gsub(/\A\//, '').gsub(/([^\/])\z/, '\1')
       end
     end
   end
